<div style"width:100%;">
  <h3>Related Documentation</h3>
  <div>
    <a href="http://cybox.readthedocs.org"><img src="{{ pathto('_static/cybox.png', 1) }}" /></a>
    <a href="http://maec.readthedocs.org"><img src="{{ pathto('_static/maec.png', 1) }}" /></a>
    <a href="http://stix.readthedocs.org"><img src="{{ pathto('_static/stix.png', 1) }}" /></a>
    <a href="http://libtaxii.readthedocs.org"><img src="{{ pathto('_static/taxii.png', 1) }}" /></a>
  </div>
</div>
