@import url("default.css");

/*
	Create styling for displaying elements side by side
	with equal horizontal space.
*/
.side-by-side {
	display: table;
	table-layout: fixed;
	
	width: 100%;
}

.side-by-side > * {
	display: table-cell;
	width: 50%;
	padding-right: 5px;
}